@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.49 2006/04/19 10:35:03 evertonm Exp $ */
+/* $Id: client.c,v 1.52 2006/04/24 14:59:18 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -123,7 +123,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   /* modifies 'session->seed' */
   tcp_check_data(sd, session, buf, rd);
 
-  nepim_session_read_add(session, rd);
+  nepim_session_read_add(session, rd + session->overhead);
 
   return OOP_CONTINUE;
 }
@@ -182,7 +182,7 @@ static void *on_tcp_write(oop_source *src, int sd,
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 
   return OOP_CONTINUE;
 }
@@ -191,7 +191,8 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
                                oop_event event, void *user)
 {
   char buf[nepim_global.tcp_write_size];
-  int wr;
+  int wr;      /* payload only */
+  int full_wr; /* payload plus overhead */
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
   int to_write;
@@ -224,7 +225,9 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 #endif /* NDEBUG */
 
   if (session->rate_remaining > 0)
-    to_write = NEPIM_MIN(session->rate_remaining, sizeof(buf));
+    to_write = NEPIM_RANGE(session->rate_remaining - session->overhead,
+			   0,
+			   sizeof(buf));
   else
     to_write = sizeof(buf);
 
@@ -270,10 +273,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(wr <= sizeof(buf));
   assert(wr <= to_write);
 
-  session->rate_remaining -= wr;
+  full_wr = wr + session->overhead;
+
+  session->rate_remaining -= full_wr;
   --session->pkt_remaining;
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, full_wr);
 
   /* finished ? */
   if ( 
@@ -421,6 +426,7 @@ static int greet(int sd)
   opt.pause_duration          = nepim_global.pause_duration;
   opt.socket_keepalive        = nepim_global.socket_keepalive;
   opt.nagle                   = nepim_global.nagle;
+  opt.overhead                = nepim_global.tcp_overhead;
 
   if (nepim_global.password) {
     opt.password_buf      = nepim_global.password;
@@ -502,7 +508,7 @@ static void *on_tcp_keepalive_write(oop_source *src, int sd,
 
   assert(wr == sizeof(buf));
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 
   /* schedule next keepalive time */
   nepim_global.oop_src->on_time(nepim_global.oop_src,
@@ -731,7 +737,6 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
   struct stat stat_buf;
   int gai_lookup; /* boolean */
   int result;
-  int nodelay;
 
   memset(&hints, 0, sizeof(hints));
 
@@ -749,9 +754,6 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       Lookup hostname
      */
 
-    nodelay = (nepim_global.nagle == -1) ?
-      -1 : !nepim_global.nagle;
-
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_protocol = IPPROTO_TCP;
     hints.ai_flags = AI_CANONNAME;
@@ -778,8 +780,6 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       Unix socket
      */
 
-    nodelay = -1; /* don't change */
-
     int len = strlen(hostname);
     if (len >= (int) sizeof(un_sock_addr.sun_path)) {
       fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
@@ -804,6 +804,7 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
+    int nodelay;
     int sd;
 
     if (ai->ai_family == PF_UNIX)
@@ -821,6 +822,8 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
     if (nepim_global.no_inet4 && (ai->ai_family == PF_INET))
       continue;
 
+    nodelay = (nepim_global.nagle == -1) ? -1 : !nepim_global.nagle;
+
     sd = nepim_connect_client_socket(ai->ai_addr, ai->ai_addrlen,
                                      ai->ai_family, ai->ai_socktype, 
                                      ai->ai_protocol,
@@ -873,6 +876,7 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       opt.random_fill             = nepim_global.random_fill;
       opt.fill_byte               = nepim_global.fill_byte;
       opt.pause_duration          = nepim_global.pause_duration;
+      opt.overhead                = nepim_global.tcp_overhead;
 
       nepim_pipe_set_add(&pipes, sd, ai->ai_addr, ai->ai_addrlen, &opt);
     }
