@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.45 2006/04/19 12:02:07 evertonm Exp $
+  $Id: main.c,v 1.47 2006/04/24 15:06:21 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -45,10 +45,14 @@ typedef void (*sighandler_t)(int);
 #include "int.h"
 #include "udp_header.h"
 
+static int UDP_OVERHEAD = 28; /* 20 ip +  8 udp */
+static int TCP_OVERHEAD = 44; /* 20 ip + 24 tcp */
+
 extern void nepim_server_run();
 extern void nepim_client_run();
 
 
+
 static void usage(FILE *out, const char *prog_name)
 {
   char buf[100];
@@ -108,6 +112,10 @@ static void usage(FILE *out, const char *prog_name)
 	  "  -F             force sending out UDP regardless of server\n"
           "  -d             duplex traffic (defaults to simplex)\n"
           "  -M             enable multicast-compatible options\n"
+	  "  -o [overhead]  protocol per-packet overhead in bytes (defaults to none)\n"
+	  "                 if the optional 'overhead' argument is ommited:\n"
+	  "                 UDP overhead defaults to %d\n"
+	  "                 TCP overhead defaults to %d\n"
           "  -c host-list   client mode (defaults to server mode)\n"
           "  -n pipes       number of parallel pipes (defaults to %d)\n"
           "  -r bit-rate    upper bit rate limit (defaults to %s bps)\n"
@@ -153,6 +161,8 @@ static void usage(FILE *out, const char *prog_name)
 	  nepim_global.tos,
 	  (long) nepim_global.udp_keepalive_recv_timer,
           nepim_global.udp_win_max,
+	  UDP_OVERHEAD,
+	  TCP_OVERHEAD,
           nepim_global.pipes,
           bit_rate,
           pkt_rate,
@@ -216,6 +226,25 @@ int main(int argc, const char *argv[])
   for (i = 1; i < argc; ++i) {
     const char *arg = argv[i];
 
+    if (!strcmp(arg, "-o")) {
+      int j = i + 1;
+      
+      if (j < argc)
+        if (*argv[j] != '-') {
+	  int overhead = atoi(argv[j]);
+          ++i;
+	  nepim_global.udp_overhead = overhead;
+          nepim_global.tcp_overhead = overhead;
+
+	  continue;
+        }
+
+      nepim_global.udp_overhead = UDP_OVERHEAD;
+      nepim_global.tcp_overhead = TCP_OVERHEAD;
+
+      continue;
+    }
+
     if (!strcmp(arg, "-N")) {
       ++i;
       if (i >= argc) {
