@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.60 2006/04/19 10:35:03 evertonm Exp $ */
+/* $Id: server.c,v 1.63 2006/04/24 14:59:18 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -107,7 +107,7 @@ static void *on_tcp_read(oop_source *src, int sd,
   /* modifies 'session->seed' */
   tcp_check_data(sd, session, buf, rd);
 
-  nepim_session_read_add(session, rd);
+  nepim_session_read_add(session, rd + session->overhead);
 
   return OOP_CONTINUE;
 }
@@ -166,7 +166,7 @@ static void *on_tcp_write(oop_source *src, int sd,
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 
   return OOP_CONTINUE;
 }
@@ -175,7 +175,8 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
                                oop_event event, void *user)
 {
   char buf[nepim_global.tcp_write_size];
-  int wr;
+  int wr;      /* payload only */
+  int full_wr; /* payload plus overhead */
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
   int to_write;
@@ -208,7 +209,9 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 #endif /* NDEBUG */
 
   if (session->rate_remaining > 0)
-    to_write = NEPIM_MIN(session->rate_remaining, sizeof(buf));
+    to_write = NEPIM_RANGE(session->rate_remaining - session->overhead,
+			   0,
+			   sizeof(buf));
   else
     to_write = sizeof(buf);
 
@@ -254,10 +257,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(wr <= sizeof(buf));
   assert(wr <= to_write);
 
-  session->rate_remaining -= wr;
+  full_wr = wr + session->overhead;
+
+  session->rate_remaining -= full_wr;
   --session->pkt_remaining;
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, full_wr);
 
   /* finished ? */
   if ( 
@@ -399,13 +404,13 @@ static int parse_greetings(int sd,
           "%d: send=%d bit_rate=%lld pkt_rate=%d "
           "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
           "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
-          "sock_ka=%d nagle=%d password=%s\n", 
+          "sock_ka=%d nagle=%d overhead=%d password=%s\n", 
           sd, opt.must_send, opt.bit_rate, opt.pkt_rate,
           opt.stat_interval, opt.test_duration, 
           opt.write_delay, opt.keepalive_must_send,
           opt.keepalive_require, opt.seed,
           opt.verify_data, opt.random_fill, opt.fill_byte, opt.pause_duration,
-	  opt.socket_keepalive, opt.nagle,
+	  opt.socket_keepalive, opt.nagle, opt.overhead,
           password_buf);
 
   if (nepim_global.password)
@@ -418,12 +423,9 @@ static int parse_greetings(int sd,
   if (nepim_socket_keepalive(sd, opt.socket_keepalive))
     return 1;
 
-  if (sockaddr_family == PF_UNIX)
-    nodelay = -1; /* dont change */
-  else
-    nodelay = (opt.nagle == -1) ? -1 : !opt.nagle;
+  nodelay = (opt.nagle == -1) ? -1 : !opt.nagle;
 
-  result = nepim_socket_tcp_opt(sd, nodelay);
+  result = nepim_socket_tcp_opt(sockaddr_family, sd, nodelay);
   if (result) {
     fprintf(stderr, 
             "%d: %s: %s: could not set tcp options: %d\n",
@@ -565,7 +567,7 @@ static void *on_tcp_keepalive_write(oop_source *src, int sd,
 
   assert(wr == sizeof(buf));
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 
   /* schedule next keepalive time */
   nepim_global.oop_src->on_time(nepim_global.oop_src,
@@ -827,7 +829,7 @@ static void *on_tcp_connect(oop_source *src, int sd,
     struct sockaddr_in6 inet6;
     struct sockaddr_un un;
   } sa;
-  int len = sizeof(sa);
+  socklen_t len = sizeof(sa);
   int result;
   int family;
   char addr_buf[500];
@@ -914,8 +916,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   struct addrinfo hints;
   struct addrinfo *ai_res;
   struct addrinfo *ai;
-  int result;
   int tcp_listeners = 0;
+  int result;
 
   memset(&hints, 0, sizeof(hints));
 
@@ -936,6 +938,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
+    int nodelay;
     int sd;
 
 #if 0
@@ -952,6 +955,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
     if (nepim_global.no_inet4 && (ai->ai_family == PF_INET))
       continue;
 
+    nodelay = (nepim_global.nagle == -1) ? -1 : !nepim_global.nagle;
+
     sd = nepim_create_listener_socket(ai->ai_addr, ai->ai_addrlen,
                                       ai->ai_family, ai->ai_socktype, 
                                       ai->ai_protocol,
@@ -960,7 +965,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
                                       nepim_global.ttl,
                                       nepim_global.win_recv,
                                       nepim_global.win_send,
-				      -1 /* tcp nodelay */,
+				      nodelay,
 				      nepim_global.tos);
     if (sd < 0) {
       fprintf(stderr, 
@@ -989,6 +994,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 static int spawn_unix_listener(const char *path)
 {
   struct sockaddr_un un_sock_addr;
+  int nodelay;
   int len;
   int sd;
 
@@ -1005,6 +1011,8 @@ static int spawn_unix_listener(const char *path)
   un_sock_addr.sun_family = AF_UNIX;
   strcpy(un_sock_addr.sun_path, path);
   
+  nodelay = (nepim_global.nagle == -1) ? -1 : !nepim_global.nagle;
+
   sd = nepim_create_listener_socket((struct sockaddr *) &un_sock_addr, 
 				    sizeof(un_sock_addr),
 				    AF_UNIX,
@@ -1015,7 +1023,7 @@ static int spawn_unix_listener(const char *path)
 				    nepim_global.ttl,
 				    nepim_global.win_recv,
 				    nepim_global.win_send,
-				    0 /* tcp nodelay */,
+				    nodelay,
 				    nepim_global.tos);
   if (sd < 0) {
     fprintf(stderr, 
