@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.56 2006/04/18 12:10:15 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.57 2006/04/23 06:33:46 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -210,7 +210,7 @@ static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
   /* any valid input traffic accounted as keepalive */
   ++session->keepalives_recv;
 
-  nepim_session_read_add(session, len);
+  nepim_session_read_add(session, len + session->overhead);
 
   if (nepim_global.udp_exp_stats)
     nepim_win_add(&slot->win_recv_seq, seq,
@@ -363,6 +363,7 @@ static int slot_greet_write(nepim_slot_t *slot)
   opt.fill_byte               = nepim_global.fill_byte;
   opt.pause_duration          = nepim_global.pause_duration;
   opt.socket_keepalive        = nepim_global.socket_keepalive;
+  opt.overhead                = nepim_global.udp_overhead;
 
   if (nepim_global.password) {
     opt.password_buf      = nepim_global.password;
@@ -606,7 +607,8 @@ static void udp_write_greet(nepim_slot_t *slot)
 static void udp_write_rate(nepim_slot_t *slot) 
 {
   char buf[nepim_global.udp_write_size];
-  int wr;
+  int wr;                   /* payload only */
+  int full_wr;              /* payload plus overhead */
   nepim_session_t *session;
   int to_write;
   int sd;
@@ -641,7 +643,7 @@ static void udp_write_rate(nepim_slot_t *slot)
 #endif /* NDEBUG */
 
   if (session->rate_remaining > 0)
-    to_write = NEPIM_RANGE(session->rate_remaining, 
+    to_write = NEPIM_RANGE(session->rate_remaining - session->overhead, 
                            UDP_HEADER_LEN, 
                            sizeof(buf));
   else
@@ -691,10 +693,12 @@ static void udp_write_rate(nepim_slot_t *slot)
 
   nepim_usock_write_good(&udp_tab, slot->udp_sd);
 
-  session->rate_remaining -= wr;
+  full_wr = wr + session->overhead;
+
+  session->rate_remaining -= full_wr;
   --session->pkt_remaining;
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, full_wr);
 
   /* finished ? */
   if ( 
@@ -716,7 +720,7 @@ static void udp_write_rate(nepim_slot_t *slot)
 static void udp_write_full(nepim_slot_t *slot) 
 {
   char buf[nepim_global.udp_write_size];
-  int wr;
+  int wr;                   /* payload only */
   nepim_session_t *session;
   int sd;
 
@@ -770,7 +774,7 @@ static void udp_write_full(nepim_slot_t *slot)
 
   nepim_usock_write_good(&udp_tab, sd);
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 }
 
 static void *on_udp_write(oop_source *src, int sd,
@@ -1261,7 +1265,7 @@ static void parse_hosts(const char *host_list)
         struct sockaddr_in6 inet6;
 	struct sockaddr_un un;
       } remote;
-      int remote_len = sizeof(remote);
+      socklen_t remote_len = sizeof(remote);
       int sd;
       nepim_greet_t opt;
 
@@ -1288,6 +1292,7 @@ static void parse_hosts(const char *host_list)
       opt.random_fill             = nepim_global.random_fill;
       opt.fill_byte               = nepim_global.fill_byte;
       opt.pause_duration          = nepim_global.pause_duration;
+      opt.overhead                = nepim_global.udp_overhead;
       
       /* create UDP slots */
       for (i = 0; i < nepim_global.pipes; ++i) {
