@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_server.c,v 1.80 2006/04/18 12:10:15 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.83 2006/04/24 14:59:18 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -369,7 +369,8 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
 static void udp_write_rate(nepim_slot_t *slot)
 {
   char buf[nepim_global.udp_write_size];
-  int wr;
+  int wr;                   /* payload only */
+  int full_wr;              /* payload plus overhead */
   nepim_session_t *session;
   int to_write;
   int sd;
@@ -403,7 +404,7 @@ static void udp_write_rate(nepim_slot_t *slot)
 #endif /* NDEBUG */
 
   if (session->rate_remaining > 0)
-    to_write = NEPIM_RANGE(session->rate_remaining, 
+    to_write = NEPIM_RANGE(session->rate_remaining - session->overhead, 
 			   UDP_HEADER_LEN, 
 			   sizeof(buf));
   else
@@ -443,10 +444,12 @@ static void udp_write_rate(nepim_slot_t *slot)
 
   nepim_usock_write_good(&udp_tab, sd);
 
-  session->rate_remaining -= wr;
+  full_wr = wr + session->overhead;
+
+  session->rate_remaining -= full_wr;
   --session->pkt_remaining;
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, full_wr);
 
   /* finished ? */
   if ( 
@@ -514,7 +517,7 @@ static void udp_write_full(nepim_slot_t *slot)
 
   nepim_usock_write_good(&udp_tab, sd);
 
-  nepim_session_write_add(session, wr);
+  nepim_session_write_add(session, wr + session->overhead);
 }
 
 static void *on_udp_write(oop_source *src, int sd,
@@ -676,6 +679,7 @@ static void schedule_udp_slot_start(int local_slot)
        Send back a packet as an ack,
        otherwise the client might exit
        with "server not responding"
+       (due to our pause delay)
     */
     char buf[nepim_global.udp_write_size];
     (void) nepim_slot_buf_sendto(slot, buf, sizeof(buf), UDP_TYPE_DATA);
@@ -746,7 +750,7 @@ static int parse_greetings(nepim_greet_t *opt,
 	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
 	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
 	  "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
-	  "sock_ka=%d password=%s\n", 
+	  "sock_ka=%d overhead=%d password=%s\n", 
 	  sd, 
 	  local_slot, remote_slot,
 	  peer,
@@ -757,6 +761,7 @@ static int parse_greetings(nepim_greet_t *opt,
 	  opt->keepalive_require, opt->seed,
 	  opt->verify_data, opt->random_fill, opt->fill_byte,
 	  opt->pause_duration, opt->socket_keepalive,
+	  opt->overhead,
 	  password_buf);
 
   if (nepim_global.tcpwrap) {
@@ -789,7 +794,7 @@ static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
   /* any valid traffic accounted as keepalive */
   ++session->keepalives_recv;
 
-  nepim_session_read_add(session, len);
+  nepim_session_read_add(session, len + session->overhead);
 
   if (nepim_global.udp_exp_stats)
     nepim_win_add(&slot->win_recv_seq, seq,
@@ -894,6 +899,7 @@ static void *on_udp_segment(oop_source *src, int sd,
 	 Send back a packet as an ack,
 	 otherwise the client might exit
 	 with "server not responding"
+	 (due to our pause delay)
       */
       {
 	nepim_slot_t *slot;
